'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var apolloServerExpress = require('apollo-server-express');
var React = _interopDefault(require('react'));
var ReactDOMServer = _interopDefault(require('react-dom/server'));
var apolloClient = require('apollo-client');
var apolloLinkSchema = require('apollo-link-schema');
var graphqlTools = require('graphql-tools');
var apolloCacheInmemory = require('apollo-cache-inmemory');
var reactHooks = require('@apollo/react-hooks');
var reactSsr = require('@apollo/react-ssr');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var LIB_TAG = "universal-react-apollo";
var REHYDRATION_STATE_DATA_KEY = "__APOLLO_STATE__";
var ROOT_CONTAINER_ID = "app-root";

var Html = function Html(_ref) {
  var content = _ref.content,
      initialState = _ref.initialState,
      headElement = _ref.headElement,
      bodyBottomElement = _ref.bodyBottomElement,
      _ref$htmlTagAttrs = _ref.htmlTagAttrs,
      htmlTagAttrs = _ref$htmlTagAttrs === void 0 ? {} : _ref$htmlTagAttrs,
      inlineStateNonce = _ref.inlineStateNonce;
  return React.createElement("html", htmlTagAttrs, React.createElement("head", null, headElement), React.createElement("body", null, React.createElement("div", {
    id: ROOT_CONTAINER_ID,
    dangerouslySetInnerHTML: {
      __html: content
    }
  }), React.createElement("script", {
    nonce: inlineStateNonce,
    dangerouslySetInnerHTML: {
      __html: "window.".concat(REHYDRATION_STATE_DATA_KEY, "=").concat(JSON.stringify(initialState).replace(/</g, "\\u003c"))
    }
  }), bodyBottomElement));
};

/**
 * Return a function which is responsible for fetching all required states via graphql operations and render application with them in server side
 * @param {Object} options Server side rendering options
 * @param {String} options.typeDefs GraphQL schema language string, array of GraphQL schema language strings or a function that return an array of GraphQL schema strings
 * @param {Object} options.resolvers GQL resolvers object - optional
 * @param {Object} options.schemaDirectives schema directive definition object
 * @return {Function} server render function
 */

function createServerRender(_ref) {
  var typeDefs = _ref.typeDefs,
      resolvers = _ref.resolvers,
      schemaDirectives = _ref.schemaDirectives;
  // create Apollo client
  var schema = graphqlTools.makeExecutableSchema({
    typeDefs: typeDefs,
    resolvers: resolvers,
    schemaDirectives: schemaDirectives
  });
  /**
   * Server render function
   * @param {Object} options
   * @param {Object} options.dataSources GQL data sources object
   * @param {Object} options.context context object which will be shared across all resolvers
   * @param {Object} options.htmlTagAttrs dom attributes of html tag
   * @param {ReactElement} options.appElement: Application main React Element
   * @param {Function} options.headElement A function called with the current request that return a React Element which will be placed in the HTML <head>
   * @param {Function} options.bodyBottomElement A function called with the current request that return a React Element which will be placed in the bottom of the HTML <body>
   * @param {Object} options.req Express request object
   * @param {Object} options.cache GQL data source cache config object - optional
   */

  return function (_ref2) {
    var dataSources = _ref2.dataSources,
        cache = _ref2.cache,
        context = _ref2.context,
        htmlTagAttrs = _ref2.htmlTagAttrs,
        appElement = _ref2.appElement,
        headElement = _ref2.headElement,
        bodyBottomElement = _ref2.bodyBottomElement,
        req = _ref2.req;

    // initialize the data source, required for Apollo RESTDataSource
    for (var _i = 0, _Object$values = Object.values(dataSources); _i < _Object$values.length; _i++) {
      var dataSource = _Object$values[_i];

      if (typeof dataSource.initialize === "function") {
        dataSource.initialize({
          context: context,
          cache: cache
        });
      }
    }

    var client = new apolloClient.ApolloClient({
      ssrMode: true,
      link: new apolloLinkSchema.SchemaLink({
        schema: schema,
        context: _objectSpread2({}, context, {
          dataSources: dataSources
        })
      }),
      cache: new apolloCacheInmemory.InMemoryCache(),
      defaultOptions: {
        watchQuery: {
          errorPolicy: "all"
        },
        query: {
          errorPolicy: "all"
        },
        mutate: {
          errorPolicy: "all"
        }
      }
    }); // wrapping main component with Apollo Provider

    var app = React.createElement(reactHooks.ApolloProvider, {
      client: client
    }, appElement);
    return reactSsr.renderToStringWithData(app).then(function (content) {
      var initialState = client.extract();
      var html = React.createElement(Html, {
        content: content,
        initialState: initialState,
        htmlTagAttrs: htmlTagAttrs,
        headElement: typeof headElement === "function" ? headElement({
          req: req
        }) : null,
        bodyBottomElement: typeof bodyBottomElement === "function" ? bodyBottomElement({
          req: req
        }) : null,
        inlineStateNonce: req.nonce
      });
      return "<!doctype html>\n".concat(ReactDOMServer.renderToStaticMarkup(html));
    });
  };
}

var initServer = function initServer(app, routes, apolloServerOptions, production) {
  // mount apollo-server-express middleware
  var apolloServer = new apolloServerExpress.ApolloServer(_objectSpread2({}, apolloServerOptions, {
    uploads: false,
    playground: !production
  }));
  apolloServer.applyMiddleware({
    app: app
  });
  var typeDefs = apolloServerOptions.typeDefs,
      resolvers = apolloServerOptions.resolvers,
      dataSources = apolloServerOptions.dataSources,
      context = apolloServerOptions.context,
      schemaDirectives = apolloServerOptions.schemaDirectives; // create server render

  var serverRender = createServerRender({
    typeDefs: typeDefs,
    resolvers: resolvers,
    schemaDirectives: schemaDirectives
  }); // mount application routing

  if (Array.isArray(routes)) {
    routes.forEach(function (_ref) {
      var path = _ref.path,
          _ref$method = _ref.method,
          method = _ref$method === void 0 ? "get" : _ref$method,
          htmlTagAttrs = _ref.htmlTagAttrs,
          appElement = _ref.appElement,
          headElement = _ref.headElement,
          bodyBottomElement = _ref.bodyBottomElement,
          _ref$middlewareChain = _ref.middlewareChain,
          middlewareChain = _ref$middlewareChain === void 0 ? [] : _ref$middlewareChain,
          _ref$responseStatusCo = _ref.responseStatusCode,
          responseStatusCode = _ref$responseStatusCo === void 0 ? 200 : _ref$responseStatusCo;
      app[method.toLowerCase()](path, middlewareChain, function (req, res, next) {
        serverRender({
          htmlTagAttrs: htmlTagAttrs,
          appElement: appElement,
          headElement: headElement,
          bodyBottomElement: bodyBottomElement,
          req: req,
          cache: apolloServer.requestOptions.cache,
          dataSources: typeof dataSources === "function" ? dataSources() : null,
          context: typeof context === "function" ? context({
            req: req,
            connection: {},
            res: res
          }) : context
        }).then(function (html) {
          res.status(responseStatusCode);
          res.send(html);
          res.end();
        })["catch"](function (error) {
          next(error);
        });
      });
    });
  } else {
    console.error("[".concat(LIB_TAG, "] Required argument - routes is not present or not an array"));
  }
};

exports.initServer = initServer;
//# sourceMappingURL=universal-react-apollo.cjs.js.map
